// Sandstorm - Personal Cloud Sandbox
// Copyright (c) 2016 Sandstorm Development Group, Inc. and contributors
// All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

"use strict";

var crypto = require("crypto");
var utils = require("../utils"),
    short_wait = utils.short_wait,
    medium_wait = utils.medium_wait,
    long_wait = utils.long_wait;

var COLLECTIONS_APP_ID = "s3u2xgmqwznz2n3apf30sm3gw1d85y029enw5pymx734cnk5n78h";
var COLLECTIONS_PACKAGE_ID = "e9408a7c077f7a9baeb9c02f0437ae40";
var COLLECTIONS_PACKAGE_URL = "https://sandstorm.io/apps/david/collections3.spk";

module.exports = {};

function setGrainTitle(browser, collectionTitle) {
  return browser
    .waitForElementVisible("#grainTitle", short_wait)
    .click("#grainTitle")
    .setAlertText(collectionTitle)
    .acceptAlert()
    .grainFrame()
    .waitForElementVisible("button[title='add description']", short_wait)
    .click("button[title='add description']")
    .waitForElementVisible("form.description-row>input[type=text]", short_wait)
    .setValue("form.description-row>input[type=text]", "This is " + collectionTitle)
    .click("form.description-row>button")
    .frame(null)
}

function powerboxCardSelector(grainId) {
  return ".popup.request .candidate-cards .powerbox-card button[data-card-id=grain-" + grainId + "]";
}

module.exports["Test Collections"] = function (browser) {
  // Prepend 'A' so that the default handle is valid.
  var devNameAlice = "A" + crypto.randomBytes(10).toString("hex");
  var devNameBob = "A" + crypto.randomBytes(10).toString("hex");
  var devNameCarol = "A" + crypto.randomBytes(10).toString("hex");

  browser
    .loginDevAccount(devNameBob)
    .executeAsync(function (done) {
      done(Meteor.userId());
    }, [], function (result) {
      var bobAccountId = result.value;
      browser.execute("window.Meteor.logout()")
        .loginDevAccount(devNameCarol)
        .executeAsync(function (done) {
          done(Meteor.userId());
        }, [], function (result) {
          var carolAccountId = result.value;
          browser.execute("window.Meteor.logout()")
            .init()
            .loginDevAccount(devNameAlice)
            .installApp(COLLECTIONS_PACKAGE_URL, COLLECTIONS_PACKAGE_ID, COLLECTIONS_APP_ID);
          browser = setGrainTitle(browser, "Collection A");

          browser.executeAsync(function (bobAccountId, done) {
            // Share Collection A to Bob.
            var grainId = Grains.findOne()._id;
            Meteor.call("newApiToken", { accountId: Meteor.userId() },
                        grainId, "petname", { allAccess: null },
                        { user: { accountId: bobAccountId, title: "Collection A", } },
                        function(error, result) {
                          done({ error: error, grainId: grainId, });
                        });
          }, [bobAccountId], function (result) {
            var grainIdA = result.value.grainId;
            browser.assert.equal(!result.value.error, true);
            browser.newGrain(COLLECTIONS_APP_ID, function (grainIdB) {
              browser = setGrainTitle(browser, "Collection B");

              browser.newGrain(COLLECTIONS_APP_ID, function (grainIdC) {
                browser = setGrainTitle(browser, "Collection C");

                browser = browser
                  .url(browser.launch_url + "/grain/" + grainIdA)
                  .grainFrame()
                  .waitForElementVisible("table.grain-list-table>tbody>tr.add-grain>td>button", medium_wait)
                  .click("table.grain-list-table>tbody>tr.add-grain>td>button")
                  .frame(null)
                  .waitForElementVisible(powerboxCardSelector(grainIdB), short_wait)
                  .click(powerboxCardSelector(grainIdB))
                  // Add with 'editor' permissions.
                  .waitForElementVisible(".popup.request .selected-card>form input[value='0']", short_wait)
                  .click(".popup.request .selected-card>form input[value='0']")
                  .click(".popup.request .selected-card>form button.connect-button")

                  .grainFrame()
                  .waitForElementVisible("table.grain-list-table>tbody>tr.add-grain>td>button", short_wait)
                  .click("table.grain-list-table>tbody>tr.add-grain>td>button")
                  .frame(null)
                  .waitForElementVisible(powerboxCardSelector(grainIdC), short_wait)
                  .click(powerboxCardSelector(grainIdC))
                  // Add with 'viewer' permissions.
                  .waitForElementVisible(".popup.request .selected-card>form input[value='1']", short_wait)
                  .click(".popup.request .selected-card>form input[value='1']")
                  .click(".popup.request .selected-card>form button.connect-button")

                  .grainFrame()
                  .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(3).grain", short_wait)
                  .click("table.grain-list-table>tbody tr:nth-child(3).grain .click-to-go")
                  .frame(null)

                  .grainFrame(grainIdC)
                  .waitForElementVisible(".description-row p", short_wait)
                  .assert.containsText(".description-row p", "This is Collection C")
                  .waitForElementVisible(".description-row button.description-button", short_wait)
                  .frame(null)

                  .execute("window.Meteor.logout()")

                  // Log in as Bob
                  .loginDevAccount(devNameBob)
                  .url(browser.launch_url + "/grain/" + grainIdA)
                  .grainFrame()
                  .waitForElementVisible(".description-row p", short_wait)
                  .assert.containsText(".description-row p", "This is Collection A")
                  .waitForElementVisible(".description-row button.description-button", short_wait)

                  .waitForElementVisible("table.grain-list-table>tbody>tr.add-grain>td>button", short_wait)
                  .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(2).grain", short_wait)
                  .assert.containsText("table.grain-list-table>tbody tr:nth-child(2).grain td>button",
                                       "Collection B")
                  .click("table.grain-list-table>tbody tr:nth-child(2).grain .click-to-go")
                  .frame(null)

                  .grainFrame(grainIdB)
                  .waitForElementVisible(".description-row p", short_wait)
                  .assert.containsText(".description-row p", "This is Collection B")
                  .waitForElementVisible(".description-row button.description-button", short_wait)

                  // As Bob, add collection A to collection B, creating a cycle of references.
                  .waitForElementVisible("table.grain-list-table>tbody>tr.add-grain>td>button", short_wait)
                  .click("table.grain-list-table>tbody>tr.add-grain>td>button")
                  .frame(null)
                  .waitForElementVisible(powerboxCardSelector(grainIdA), short_wait)
                  .click(powerboxCardSelector(grainIdA))
                  // Add with 'viewer' permissions.
                  .waitForElementVisible(".popup.request .selected-card>form input[value='1']", short_wait)
                  .click(".popup.request .selected-card>form input[value='1']")
                  .click(".popup.request .selected-card>form button.connect-button")

                  // Navigate back to collection A by clicking on it in collection B.
                  .grainFrame()
                  .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(2).grain", short_wait)
                  .click("table.grain-list-table>tbody tr:nth-child(2).grain .click-to-go")
                  .frame(null)

                  .grainFrame(grainIdA)
                  .waitForElementVisible("table.grain-list-table>tbody>tr.add-grain>td>button", short_wait)
                  .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(3).grain", short_wait)
                  .assert.containsText("table.grain-list-table>tbody tr:nth-child(3).grain td>button",
                                       "Collection C")
                  .click("table.grain-list-table>tbody tr:nth-child(3).grain .click-to-go")
                  .frame(null)

                  .grainFrame(grainIdC)
                  .waitForElementVisible(".description-row p", short_wait)
                  .assert.containsText(".description-row p", "This is Collection C")
                  .assert.elementNotPresent(".description-row button.description-button")

                  .frame(null)
                  .executeAsync(function (carolAccountId, grainIdB, done) {
                    // As Bob, share Collection B to Carol.
                    Meteor.call("newApiToken", { accountId: Meteor.userId() },
                        grainIdB, "petname", { allAccess: null },
                        { user: { accountId: carolAccountId, title: "Collection B", } },
                        function(error, result) {
                          done({ error: error });
                        });
                  }, [carolAccountId, grainIdB], function (result) {
                    browser.assert.equal(!result.value.error, true);
                    browser
                      .execute("window.Meteor.logout()")

                      // Log in as Carol
                      .loginDevAccount(devNameCarol)
                      .url(browser.launch_url + "/grain/" + grainIdB)
                      .grainFrame()
                      .waitForElementVisible(".description-row p", short_wait)
                      .assert.containsText(".description-row p", "This is Collection B")
                      .waitForElementVisible(".description-row button.description-button", short_wait)

                      .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(2).grain",
                                             short_wait)
                      .assert.containsText("table.grain-list-table>tbody tr:nth-child(2).grain td>button",
                                           "Collection A")
                      .click("table.grain-list-table>tbody tr:nth-child(2).grain .click-to-go")

                      .grainFrame(grainIdA)

                      .waitForElementVisible(".description-row p", short_wait)
                      .assert.containsText(".description-row p", "This is Collection A")

                      // Carol does not have edit permissions.
                      .assert.elementNotPresent(".description-row button.description-button")
                      .frame(null)

                      .execute("window.Meteor.logout()")

                      // Log back in as Alice
                      .loginDevAccount(devNameAlice)
                      .url(browser.launch_url + "/grain/" + grainIdA)
                      .disableGuidedTour()
                      .grainFrame()

                      // Unlink collection B from collection A.
                      .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(3).grain",
                                             short_wait)
                      .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(2).grain",
                                             short_wait)
                      .assert.containsText("table.grain-list-table>tbody tr:nth-child(2).grain td>button",
                                           "Collection B")
                      .click("table.grain-list-table>tbody tr:nth-child(2).grain td>input[type=checkbox]")
                      .waitForElementVisible(".bulk-action-buttons>button[title='unlink selected grains']",
                                             short_wait)
                      .click(".bulk-action-buttons>button[title='unlink selected grains']")

                      .frame(null)
                      .execute("window.Meteor.logout()")

                      // Log back in as Carol. Check that she can no longer access collection A.
                      .loginDevAccount(devNameCarol)

                      // Add some characters onto the end of the URL because otherwise we trigger
                      // the grain-tab restore logic and tabs open for both Collection A and Collection B.
                      .url(browser.launch_url + "/grain/" + grainIdA + "/#")

                      .waitForElementVisible(".grain-interstitial.request-access", medium_wait)
                      .end();
                  });
              });
            });
          });
        });
    });
};

module.exports["Test collections anonymous user"] = function (browser) {
  browser = browser
    .init()
    .loginDevAccount()
    .installApp(COLLECTIONS_PACKAGE_URL, COLLECTIONS_PACKAGE_ID, COLLECTIONS_APP_ID);
  browser = setGrainTitle(browser, "Collection A");
  browser.executeAsync(function (done) {
    var grainId = Grains.findOne()._id;
    Meteor.call("newApiToken", { accountId: Meteor.userId() },
                grainId, "petname", { allAccess: null },
                { webkey: { forSharing: true }, },
                function(error, result) {
                  done({ error: error, grainId: grainId, token: (result || {}).token });
                });
  }, [], function (result) {
    var grainIdA = result.value.grainId;
    var tokenA = result.value.token;
    browser.assert.equal(!result.value.error, true);
    browser.newGrain(COLLECTIONS_APP_ID, function (grainIdB) {
      browser = setGrainTitle(browser, "Collection B");

      browser
        .url(browser.launch_url + "/grain/" + grainIdA)
        .grainFrame()
        .waitForElementVisible("table.grain-list-table>tbody>tr.add-grain>td>button", medium_wait)
        .click("table.grain-list-table>tbody>tr.add-grain>td>button")
        .frame(null)
        .waitForElementVisible(powerboxCardSelector(grainIdB), short_wait)
        .click(powerboxCardSelector(grainIdB))

        // Add with 'editor' permissions.
        .waitForElementVisible(".popup.request .selected-card>form input[value='0']", short_wait)
        .click(".popup.request .selected-card>form input[value='0']")
        .click(".popup.request .selected-card>form button.connect-button")

        // Visit token A anonymously. The link should still work.
        .frame(null)
        .execute("window.Meteor.logout()")

        .url(browser.launch_url + "/shared/" + tokenA)
        .waitForElementVisible(".popup.login button.close-popup", short_wait)
        .click(".popup.login button.close-popup")
        .grainFrame()
        .waitForElementVisible(".description-row p", short_wait)
        .assert.containsText(".description-row p", "This is Collection A")
        .waitForElementVisible(".description-row button.description-button", short_wait)
        .waitForElementVisible("table.grain-list-table>tbody tr:nth-child(1).grain", short_wait)
        .assert.containsText("table.grain-list-table>tbody tr:nth-child(1).grain td>button",
                             "Collection B")
        .click("table.grain-list-table>tbody tr:nth-child(1).grain .click-to-go")
        .frame(null)

        .grainFrame(grainIdB)
        .waitForElementVisible(".description-row p", short_wait)
        .assert.containsText(".description-row p", "This is Collection B")
        .waitForElementVisible(".description-row button.description-button", short_wait)

        .frame(null)
        .url(function (sharedUrlGrainB) {
          browser.loginDevAccount()
            .url(sharedUrlGrainB.value)
            .waitForElementVisible(".grain-interstitial button.reveal-identity-button", short_wait)
            .click(".grain-interstitial button.reveal-identity-button")
            .grainFrame(grainIdB)
            .end();
        });
    });
  });
}
